#!/bin/sh
VER=1.0
#-------------------------------------------------------------------#
# Tur-DisableUser 1.0. A rewrite of the popular enable/disable      #
# script, made ages ago. With this you can prevent users from       #
# logging on. You can trigger it from irc, shell or from glftpd.    #
# You can of course enable them again as well as get a full list of #
# all the disabled users.                                           #
# There is no reason to worry about them being accidently purged    #
# while they are disabled.                                          #
#-------------------------------------------------------------------#
# Installation:                                                     #
# Put tur-disableuser.sh in /glftpd/bin                             #
# Put tur-disableuser.tcl in the bots scripts folder and add it to  #
#  the bots .conf file.                                             #
# Make both scripts executable (chmod 755, or similar).             #
# Edit the settings below in this script.                           #
# Edit your /etc/glftpd.conf file. Change your current shutdown     #
# line to: shutdown !J *                                            # 
#                                                                   #
#  Explanation: What we really do is give them flag J. That is a    #
#  custom flag and should not be used for anything else really. It  #
#  can be changed in the options if you use it for something else.  #
#  Anyway, when changing shutdown as described, anyone but those    #
#  with flag J can log in. Those who are disabled with flag J get   #
#  "The server is down for maintainance" or whatever it says in     #
#  your glstrings.                                                  #
#  If you want to close the server, only change the *               #
#  shutdown !J !*  = Nobody can login.                              #
#  shutdown !J 1   = Only siteops can log in.                       #
#                                                                   #
# The default trigger in irc and glftpd for this is 'disuser' so    #
# add this to glftpd.conf as well:                                  #
#  site_cmd disuser        EXEC    /bin/tur-disableuser.sh          #
#  custom-disuser          1                                        #
# It can, ofcourse, be changed to anything else instead of difuser. #
#                                                                   #
# If you edit the .tcl, you will see the same thing. There will be  #
# two !disuser here. The first is for open channel and the other is #
# for private message to the bot ( so you can do it incognito ).    #
# They should use the same triggers ( Are you there soehest? =))    #
# By default, its locked to channel ops only.                       #
#                                                                   #
# You may also execute the .sh from shell directly.                 #
#                                                                   #
# Thats it. Rehash the bot and try !disuser from irc and            #
# 'site disuser' from inside glftpd.                                #
#                                                                   #
# If you, when running it from the bot, get an error like           #
# /root/.bashrc, access denied, then fix it or do this:             #
# chmod 755 /root; chmod 755 /root/.bashrc                          #
# I havent been able to figure out why really.. Lazy bum.           #
#                                                                   #
# Oh, one more thing. The argument 'status' will give you a list of #
# all disabled users.                                               #
#-------------------------------------------------------------------#
# Contact Turranius on efnet at #glftpd                             #
# http://www.grandis.nu/glftpd/  >  http://grandis.mine.nu/glftpd/  #
#-------------------------------------------------------------------#

## Root of glftpd
glroot=/glftpd

## Users folder, relative to glroot above
users=/ftp-data/users

## A temporary folder to write to, relative to glroot above.
tmp=/tmp

## Which custom flag to use. If you use J for something else
## then change it to whatever you like. Only use one of the
## custom flags described in glftpd.docs.
customflag=J

## What perms to set the userfile to once finished. "" = disable.
perms=777


#################################################################################
# No changes below here unless you wanna change some text or something...       #
#################################################################################

if [ "$FLAGS" = "" ]; then
  users="$glroot$users"
  tmp="$glroot$tmp"
fi

if [ "$1" = "" -a "$2" = "" -o "$1" != "" -a "$2" = "" -a "$1" != "status" ]; then
  echo "Select to 'disable' or 'enable' a user. Also enter user to do it on. Give argument 'status' for list of disabled users."
  exit 0
fi

proc_checkexist() {
  if [ ! -e $users/$user ]; then
    echo "User $user does not exist..."
    exit 0
  fi
  if [ ! -w $users/$user ]; then
    echo "Error. $user found but no permissions to write to userfile. Fix perms."
    exit 0
  fi
}

proc_disable() {
  flagsnow="$( grep -w FLAGS $users/$user | grep -v 'TAGLINE'| awk '{print $2}')"
  verify="$( echo $flagsnow | grep $customflag )"
  if [ "$verify" != "" ]; then
    echo "Error: User $user has already been disabled."
    exit 0
  else
    flagsafter=$flagsnow$customflag
    sed -e "s/^FLAGS $flagsnow/FLAGS $flagsafter/" $users/$user > $tmp/$user.disable
    if [ -e $tmp/$user.disable ]; then
      mv -f $tmp/$user.disable $users/$user
    else
      echo "Error: Could not create $user.disable in the tmp dir. Fix perms."
      exit 0
    fi
    if [ "$perms" != "" ]; then
      chmod $perms $users/$user
    fi
    echo "User $user has been denied access to login."
    exit 0
  fi
}

proc_enable() {
  flagsnow="$( grep -w FLAGS $users/$user | grep -v 'TAGLINE'| awk '{print $2}')"
  verify="$( echo $flagsnow | grep $customflag )"
  if [ "$verify" = "" ]; then
    echo "Error: User $user has not been disabled."
    exit 0
  else
    flagsafter="$( echo $flagsnow | tr -d "$customflag" )"
    sed -e "s/^FLAGS $flagsnow/FLAGS $flagsafter/" $users/$user > $tmp/$user.enable
    if [ -e $tmp/$user.enable ]; then
      mv -f $tmp/$user.enable $users/$user
    else
      echo "Error: Could not create $user.enable in the tmp dir. Fix perms."
      exit 0
    fi
    if [ "$perms" != "" ]; then
      chmod $perms $users/$user
    fi
    echo "User $user has been granted login access again."
    exit 0
  fi
}

proc_status() {
  for each in `grep -w "FLAGS" $users/* | grep $customflag | awk '{print $1"^"$2}'`; do
    user="$( echo $each | awk -F":" '{print $1}' )"
    user="$( basename $user )"
    flags="$( echo $each | awk -F"^" '{print $2}' )"
    verify="$( echo $flags | grep $customflag )"
    if [ "$verify" != "" ]; then
      if [ "$disabledusers" = "" ]; then
        disabledusers="$user"
        gotone="yes"
      else
        disabledusers="$user $disabledusers"
      fi
    fi
  done
  if [ "$gotone" = "" ]; then
    echo "No users disabled that I can see."
  else
    echo "Currently disabled users: $disabledusers"
  fi
  exit 0
}  

user="$2"

case $1 in
  enable) proc_checkexist; proc_enable;;
  disable) proc_checkexist; proc_disable;;
  status) proc_status;;
  *) echo "Error. This shouldnt have happened."; exit 0;;
esac

exit 0  


